_G.meth_mastermind = {}
meth_mastermind._id = "Meth Mastermind"
meth_mastermind._path = ModPath
meth_mastermind._save_path = SavePath .. "meth_mastermind.json"
meth_mastermind._data = {
	prevent_wrong_ingredient = true,
	green_contour = true,
	prevent_explosion = true,
	chat = 1
}

function meth_mastermind:save()
	local file = io.open(self._save_path, 'w+')
	
	if file then
		file:write(json.encode(self._data))
		file:close()
	end
end

function meth_mastermind:load()
	local file = io.open(self._save_path, "r")

	if file then
		self._data = json.decode(file:read("*all"))
		file:close()
	end
end

local menu_id = meth_mastermind._id

Hooks:Add("MenuManagerSetupCustomMenus", meth_mastermind._id .. " --> MenuManagerSetupCustomMenus", function()
	MenuHelper:NewMenu(menu_id)
end)

Hooks:Add("LocalizationManagerPostInit", meth_mastermind._id .. " --> LocalizationManagerPostInit", function(loc)
	loc:load_localization_file(meth_mastermind._path .. "loc/en.json")
end)

Hooks:Add("MenuManagerBuildCustomMenus", meth_mastermind._id .. " --> MenuManagerBuildCustomMenus", function(_, nodes)
	nodes[menu_id] = MenuHelper:BuildMenu(menu_id)
	MenuHelper:AddMenuItem(nodes[BLT.Mods.Constants:LuaModOptionsMenuID()], menu_id, "meth_mastermind_menu_name", "meth_mastermind_menu_desc")
end)

function MenuCallbackHandler:meth_mastermind_toggle_callback(item)
	local selected_item = item._parameters.name:match("meth_mastermind_(.-)_toggle")
	local enabled = item.selected == 1
	
	meth_mastermind._data[selected_item] = enabled
	meth_mastermind:save()
end

function MenuCallbackHandler:meth_mastermind_slider_callback(item)
	local selected_item = item._parameters.name:match("meth_mastermind_(.-)_slider")
	
	meth_mastermind._data[selected_item] = item._current_index
	meth_mastermind:save()
end

Hooks:Add("MenuManagerPopulateCustomMenus", meth_mastermind._id .. " --> MenuManagerPopulateCustomMenus", function(_, nodes)
	meth_mastermind:load()
	
	local priority = math.huge

	MenuHelper:AddToggle({
		id = "meth_mastermind_prevent_wrong_ingredient_toggle",
		title = "meth_mastermind_prevent_wrong_ingredient_toggle_name",
		desc = "meth_mastermind_prevent_wrong_ingredient_toggle_desc",
		callback = "meth_mastermind_toggle_callback",
		value = meth_mastermind._data.prevent_wrong_ingredient,
		menu_id = menu_id,
		priority = priority
	})
	
	priority = priority - 1
	
	MenuHelper:AddToggle({
		id = "meth_mastermind_green_contour_toggle",
		title = "meth_mastermind_green_contour_toggle_name",
		desc = "meth_mastermind_green_contour_toggle_desc",
		callback = "meth_mastermind_toggle_callback",
		value = meth_mastermind._data.green_contour,
		menu_id = menu_id,
		priority = priority
	})
	
	priority = priority - 1

	MenuHelper:AddToggle({
		id = "meth_mastermind_prevent_explosion_toggle",
		title = "meth_mastermind_prevent_explosion_toggle_name",
		desc = "meth_mastermind_prevent_explosion_toggle_desc",
		callback = "meth_mastermind_toggle_callback",
		value = meth_mastermind._data.prevent_explosion,
		menu_id = menu_id,
		priority = priority
	})
	
	priority = priority - 1
	
	MenuHelper:AddMultipleChoice({
		id = "meth_mastermind_chat_slider",
		title = "meth_mastermind_chat_slider_name",
		desc = "meth_mastermind_chat_slider_desc",
		callback = "meth_mastermind_slider_callback",
		items = {"meth_mastermind_chat_slider_1", "meth_mastermind_chat_slider_2"},
		value = meth_mastermind._data.chat,
		menu_id = menu_id,
		priority = priority
	})

	meth_mastermind:save()
end)