if not meth_mastermind then
	return
end

meth_mastermind._chat_prefix = "[" .. meth_mastermind._id .. "] "
meth_mastermind.correct_ingredient = nil
meth_mastermind.ingredients_used_amount = 0
meth_mastermind.chat_ingredients_used_amount = nil
meth_mastermind.lab_units = {}

local ingredients = {
	"acid",
	"caustic_soda",
	"hydrogen_chloride"
}

local dialogs = {
	"pln_rt1_20",
	"pln_rt1_22",
	"pln_rt1_24",
	"pln_rat_stage1_20",
	"pln_rat_stage1_22",
	"pln_rat_stage1_24",
	"Play_loc_mex_cook_03",
	"Play_loc_mex_cook_04",
	"Play_loc_mex_cook_05"
}

local dialog_ids = {}

for i = 1, #dialogs, 3 do
    for j, ingredient in ipairs(ingredients) do
		dialog_ids[dialogs[i + j - 1]] = ingredient
	end
end

local since_ruined_msg = 0

local function announce_ingredient()
	if meth_mastermind.ingredients_used_amount == meth_mastermind.chat_ingredients_used_amount then
		return
	end
	
	meth_mastermind.chat_ingredients_used_amount = meth_mastermind.ingredients_used_amount

	local msg = meth_mastermind.correct_ingredient:gsub("_", " ")
	msg = msg:sub(1, 1):upper() .. msg:sub(2)
	
	local peer = managers.network:session():local_peer()
	
	if meth_mastermind._data.chat == 2 and peer then
		managers.chat:_receive_message(ChatManager.GAME, meth_mastermind._chat_prefix, msg, tweak_data.chat_colors[peer:id()])
	end
end

local function prevented_explosion(peer, ingredient)
	if not peer or os.time() - since_ruined_msg < 10 then
		return
	end
	
	local local_peer = managers.network:session():local_peer()
	local is_local_peer = local_peer == peer
	
	if not is_local_peer then
		peer:send("give_equipment", ingredient, 1, true)
	end
	
	local color_name = "Blue"
	
	if peer:id() == 3 then
		color_name = "Red"
	elseif peer:id() == 4 then
		color_name = "Orange"
	end
	
	if is_local_peer then
		color_name = "You"
	end
	
	local msg = color_name .. " nearly ruined " .. (is_local_peer and "the" or "my") .. " lab! The correct ingredient is " .. meth_mastermind.correct_ingredient:gsub("_", " ")
	
	if is_local_peer then
		managers.chat:_receive_message(ChatManager.GAME, meth_mastermind._chat_prefix, msg, tweak_data.chat_colors[peer:id()])
	else
		managers.chat:send_message(ChatManager.GAME, nil, meth_mastermind._chat_prefix .. msg)
	end
	
	since_ruined_msg = os.time()
end

local function predict_next(i)
	meth_mastermind.ingredients_used_amount = meth_mastermind.ingredients_used_amount + 1
	meth_mastermind.correct_ingredient = nil
	
	if managers.job:current_level_id() == "mia_1" then
		meth_mastermind.correct_ingredient = i and i + 1 > #ingredients and ingredients[1] or ingredients[i + 1]
	end
end

if BaseInteractionExt then
	local BaseInteractionExt_can_select = BaseInteractionExt.can_select

	function BaseInteractionExt:can_select(...)
		local id = self._unit:interaction()._tweak_data.special_equipment
		
		if meth_mastermind._data.prevent_wrong_ingredient then
			for _, ingredient in ipairs(ingredients) do
				if ingredient == id and ingredient ~= meth_mastermind.correct_ingredient then
					return
				end
			end
		end
		
		return BaseInteractionExt_can_select(self, ...)
	end
	
	local BaseInteractionExt_interact_start = BaseInteractionExt.interact_start

	function BaseInteractionExt:interact_start(...)
		local id = self._unit:interaction()._tweak_data.special_equipment
		
		if meth_mastermind._data.prevent_explosion and meth_mastermind.correct_ingredient then
			for i, ingredient in ipairs(ingredients) do
				if ingredient == id and ingredient ~= meth_mastermind.correct_ingredient then
					prevented_explosion(managers.network:session():local_peer(), ingredient)
					return
				end
			end
		end
		
		return BaseInteractionExt_interact_start(self, ...)
	end
	
	local BaseInteractionExt_interact = BaseInteractionExt.interact

	function BaseInteractionExt:interact(...)
		BaseInteractionExt_interact(self, ...)
		
		if not meth_mastermind.correct_ingredient then
			return
		end
		
		local id = self._unit:interaction()._tweak_data.special_equipment
			
		for i, ingredient in ipairs(ingredients) do
			if ingredient == id and ingredient == meth_mastermind.correct_ingredient then
				predict_next(i)
				break
			end
		end
	end
	
	local BaseInteractionExt_set_contour = BaseInteractionExt.set_contour

	function BaseInteractionExt:set_contour(color, ...)
		local interaction = self._unit:interaction()
		local id = interaction._tweak_data.special_equipment
		
		if not meth_mastermind.lab_units[self._unit] then
			for _, ingredient in ipairs(ingredients) do
				if id == ingredient then
					meth_mastermind.lab_units[self._unit] = true
					break
				end
			end
		end
		
		if meth_mastermind.lab_units[self._unit] then
			if managers.job:current_level_id() == "mia_1" and not meth_mastermind.correct_ingredient then
				meth_mastermind.correct_ingredient = ingredients[1]
			end
			
			local contour_color = tweak_data.contour[self._tweak_data.contour or "interactable"][color or "standard_color"]
			
			if meth_mastermind._data.green_contour then
				contour_color = id == meth_mastermind.correct_ingredient and Vector3(0, 1, 0) or contour_color
			end
			
			for _, m in ipairs(interaction._materials) do
				m:set_variable(Idstring("contour_color"), contour_color)
				m:set_variable(Idstring("contour_opacity"), self._active and 1 or 0)
			end
			
			if interaction._active and id == meth_mastermind.correct_ingredient then
				announce_ingredient()
			end
			
			return
		end
		
		BaseInteractionExt_set_contour(self, color, ...)
	end
end

if DialogManager then
	Hooks:PostHook(DialogManager, "queue_dialog", meth_mastermind._id .. " --> DialogManager:queue_dialog", function(_, id)
		meth_mastermind.correct_ingredient = dialog_ids[id] or meth_mastermind.correct_ingredient
		
		if not meth_mastermind.correct_ingredient then
			return
		end
		
		for unit in pairs(meth_mastermind.lab_units) do
			local id = alive(unit) and unit:interaction()._tweak_data.special_equipment

			if id == meth_mastermind.correct_ingredient then
				unit:interaction():set_contour()
			end
		end
	end)
end

if UnitNetworkHandler then
	local UnitNetworkHandler_sync_interacted = UnitNetworkHandler.sync_interacted

	function UnitNetworkHandler:sync_interacted(unit, ...)
		local id = unit and unit:interaction() and unit:interaction()._tweak_data and unit:interaction()._tweak_data.special_equipment
		
		if Network:is_server() and id and meth_mastermind.correct_ingredient then
			for i, ingredient in ipairs(ingredients) do
				if ingredient == id then
					if ingredient == meth_mastermind.correct_ingredient then
						predict_next(i)
						break
					elseif meth_mastermind._data.prevent_explosion then
						local sender = select(select("#", ...), ...)
						local peer = sender and self._verify_sender(sender)
	
						if peer then
							prevented_explosion(peer, ingredient)
						end
						
						return
					end
				end
			end
		end
		
		UnitNetworkHandler_sync_interacted(self, unit, ...)
	end
end